/*+-------------------------------------------------------------------------
	ecushm.c - shared memory handler for ecu xmtr/rcvr comm
	wht@n4hgf.Mt-Park.GA.US

  Defined functions:
	shm_done()
	shm_init()
	shmr_process_rcvr_SIGUSR2()
	shmr_set_xmtr_bn_1()
	shmx_connect()
	shmx_make_rcvr_sleep(seconds)
	shmx_process_xmtr_SIGUSR2()
	shmx_rc_report(prcvd_chars,prcvd_chars_this_connect)
	shmx_set_rcvr_ansi(mode)
	shmx_set_rcvr_bn(bn_value)
	shmx_set_rcvr_log(logfilename,append_flag,raw_flag)
	shmx_set_rcvr_nl_control()

--------------------------------------------------------------------------*/
/*+:EDITS:*/
/*:07-25-1991-12:56-wht@n4hgf-ECU release 3.10 */
/*:08-14-1990-20:40-wht@n4hgf-ecu3.00-flush old edit history */

#include "ecu.h"
#include <sys/ipc.h>
#include <sys/shm.h>

/* xmtr to rcvr cmds */
#define SHMX_MAKE_RCVR_SLEEP	1	/* sleep xi1=#seconds */
#define	SHMX_SET_RCVR_BN		2	/* set want_bell_notify xi1=value */
#define SHMX_SET_RCVR_LOG		3	/* log file manipulations
									 * xi1=append,xi2=raw,xs1=name */
/* rcvr to xmtr cmds */
#define SHMR_SET_XMTR_BN1		1	/* set want_bell_notify to 1 */

extern int want_bell_notify;
extern char rcvr_log_file[];	/* if rcvr_log!= 0,log filename */
extern int rcvr_log;			/* rcvr log active if != 0 */
extern FILE *rcvr_log_fp;		/* rcvr log file */
extern int rcvr_log_raw;		/* if true, log all, else filter ctl chrs */
extern int rcvr_log_flusheach;
extern int rcvr_log_append;	/* if true, append, else scratch */
extern int rcvr_log_gen_title;

ECU_SDS FAR *shm;				/* shared segment pointer */
key_t shm_key;
int shm_shmid;

/*+-------------------------------------------------------------------------
	shm_init()
  Called by parent process (xmtr) to initialize environment
--------------------------------------------------------------------------*/
shm_init()
{
uint save_LINESxCOLS;
extern uint LINESxCOLS;

	shm_key = 0xEC000000L | getpid();

	if((shm_shmid = shmget(shm_key,sizeof(ECU_SDS),IPC_CREAT | 0600)) < 0)
	{
		perror("shmget");
		exit(11);
	}

	if((shm = (ECU_SDS FAR *)shmat(shm_shmid,(char FAR *)0,0)) ==
		(ECU_SDS FAR *)-1)
	{
		perror("shmat");
		exit(11);
	}

	shm->xcmd = 0;	/* signal from xmtr to rcvr SIGUSR2 */
	shm->xi1 = 0;	/* parameters */
	shm->xi2 = 0;
	shm->xs1[0] = 0;
	shm->rcmd = 0;	/* signal from rcvr to xmtr SIGUSR2 */
	shm->ri1 = 0;	/* parameters */
	shm->ri2 = 0;
	shm->rs1[0] = 0;
	shm->rcvd_chars = 0L;
	shm->rcvd_chars_this_connect = 0L;

	shm->cursor_y = 0;
	shm->cursor_x = 0;
	shm->friend_space[0] = 0;
	save_LINESxCOLS = LINESxCOLS;
	LINESxCOLS = sizeof(shm->screen);	/* avoid trap */
	spaces((char *)shm->screen,sizeof(shm->screen));
	LINESxCOLS = save_LINESxCOLS;

	return(0);

}	/* end of shm_init */

/*+-------------------------------------------------------------------------
	shm_done() -- finished with shm -- clean up
--------------------------------------------------------------------------*/
void
shm_done()
{
	if(shmctl(shm_shmid,IPC_RMID,(struct shmid_ds *)0))
		perror("shmctl IPC_RMID");
}	/* end of shm_done */

/*+-------------------------------------------------------------------------
	shmx_rc_report(prcvr_chars,prcvr_chars_this_connect)
 xmtr calls to get rcvr stats
--------------------------------------------------------------------------*/
void
shmx_rc_report(prcvd_chars,prcvd_chars_this_connect)
long *prcvd_chars;
long *prcvd_chars_this_connect;
{
	*prcvd_chars = shm->rcvd_chars;
	*prcvd_chars_this_connect = shm->rcvd_chars_this_connect;
}	/* end of shmx_rc_report */

/*+-------------------------------------------------------------------------
	shmx_connect() -- xmtr calls when modem connects
--------------------------------------------------------------------------*/
void
shmx_connect()
{
	shm->rcvd_chars_this_connect = 0L;
}	/* end of shmx_connect */

/*+-------------------------------------------------------------------------
	shmx_make_rcvr_sleep(seconds)
--------------------------------------------------------------------------*/
void
shmx_make_rcvr_sleep(seconds)
int seconds;
{
	shm->xcmd = SHMX_MAKE_RCVR_SLEEP;
	shm->xi1 = seconds;
	kill_rcvr_process(SIGUSR2);

}	/* end of shmx_make_rcvr_sleep */

/*+-------------------------------------------------------------------------
	shmx_set_rcvr_log(logfilename,append_flag,raw_flag,flush_each)

null logfilename stops logging
append_flag says whether to open for write or append
raw_flag says whether or not to filter non-printable chars or not
(NL not filtered)
--------------------------------------------------------------------------*/
void
shmx_set_rcvr_log(logfilename,append_flag,raw_flag,flush_each)
char *logfilename;
int append_flag;
int raw_flag;
int flush_each;
{
	shm->xcmd = SHMX_SET_RCVR_LOG;
	shm->xi1 = append_flag;
	shm->xi2 = raw_flag;
	shm->xi3 = flush_each;
	strcpy(shm->xs1,logfilename);
	kill_rcvr_process(SIGUSR2);
}	/* end of shmx_set_rcvr_log */

/*+-------------------------------------------------------------------------
	shmx_set_rcvr_bn(bn_value)
--------------------------------------------------------------------------*/
void
shmx_set_rcvr_bn(bn_value)
int bn_value;
{
	shm->xcmd = SHMX_SET_RCVR_BN;
	shm->xi1 = bn_value;
	kill_rcvr_process(SIGUSR2);

}	/* end of shmx_set_rcvr_bn */

/*+-------------------------------------------------------------------------
	shmr_set_xmtr_bn_1()
--------------------------------------------------------------------------*/
void
shmr_set_xmtr_bn_1()
{
	shm->rcmd = SHMR_SET_XMTR_BN1;
	kill(xmtr_pid,SIGUSR2);

}	/* end of shmr_set_xmtr_bn_1 */

/*+-------------------------------------------------------------------------
	shmx_process_xmtr_SIGUSR2()
--------------------------------------------------------------------------*/
void
shmx_process_xmtr_SIGUSR2()
{
register rcmd;
register ri1,ri2;

	rcmd = shm->rcmd;
	shm->rcmd = 0;
	ri1 = shm->ri1;
	ri2 = shm->ri2;

	switch(rcmd)
	{
		case SHMR_SET_XMTR_BN1:
			want_bell_notify = 1;
			break;
	}

}	/* end of shmx_process_xmtr_SIGUSR2 */

/*+-------------------------------------------------------------------------
	shmr_process_rcvr_SIGUSR2()
--------------------------------------------------------------------------*/
void
shmr_process_rcvr_SIGUSR2()
{
register xcmd;
register xi1,xi2,xi3;
char xs1[SHM_STRLEN];

	xcmd = shm->xcmd;
	shm->xcmd = 0;
	xi1 = shm->xi1;
	xi2 = shm->xi2;
	xi3 = shm->xi3;
	strcpy(xs1,shm->xs1);

	switch(xcmd)
	{
		case SHMX_MAKE_RCVR_SLEEP:
			sleep(xi1);
			break;

		case SHMX_SET_RCVR_BN:
			want_bell_notify = xi1;
			break;

		case SHMX_SET_RCVR_LOG:
			if(rcvr_log) 		/* if already logging */
			{
				if(!rcvr_log_raw)
					LOGPUTC('\n',rcvr_log_fp);
				fclose(rcvr_log_fp);
				rcvr_log = 0;
			}
			if(strlen(xs1) == 0)	/* if all we wanted was to stop log ... */
				break;				/* ... then quit */
			rcvr_log_gen_title = 1;
			rcvr_log = 1;
			rcvr_log_append = xi1;
			rcvr_log_raw = xi2;
			rcvr_log_flusheach = xi3;
			strcpy(rcvr_log_file,xs1);
			rcvr_log_open();
			break;
	}

}	/* end of shmr_process_rcvr_SIGUSR2 */

/* end of ecushm.c */
/* vi: set tabstop=4 shiftwidth=4: */
